@@ -2,10 +2,10 @@
  *  cache-file.c
  *  mod_musicindex
  *
- *  $Id: cache-file.c 670 2006-07-31 21:34:34Z varenet $
+ *  $Id: cache-file.c 754 2007-04-29 00:57:53Z varenet $
  *
  *  Created by Thibaut VARENE on Wed Feb 23 2005.
- *  Copyright (c) 2003-2006 Thibaut VARENE
+ *  Copyright (c) 2003-2007 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -18,8 +18,8 @@
  * Flat file cache management subsystem.
  *
  * @author Thibaut Varene
- * @version $Revision: 670 $
- * @date 2003-2006
+ * @version $Revision: 754 $
+ * @date 2003-2007
  *
  * @warning 'errno' seems not to be seen as volatile by the compiler, therefore we
  *	cannot do "if(mkdir(foo) && errno == EEXIST)" for instance.
@@ -49,8 +49,11 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>	/* file handling */
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>	/* strerror() */
+#endif
 
-#define CACHE_VERS	9
+#define CACHE_VERS	10
 #define CACHE_NF	15
 
 #define CA_OK		0
@@ -141,7 +144,7 @@ static void error_handler(request_rec *r, const char *const caller)
 			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) No space left on device!", caller);
 			break;
 		default:
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) - error_handler! errno=%i", caller, errno);
+			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) - error_handler! %s", caller, strerror(errno));
 			break;
 	}
 	return;
@@ -268,20 +271,24 @@ error_out:
 /**
  * Checks if a directory already exists in the cache.
  *
- * This function is called when the caller wants to know whether a given
- * directory exists or not in the cache.
- * If the directory exists, the function returns successfully, otherwise
- * it tries to create it if possible and returns accordingly.
+ * This function takes advantage of the standard behaviour of the cache
+ * backend in musicindex: this function is called when the caller wants to know
+ * whether the backend handles a given directory. We always return NULL,
+ * pretending we don't, as "handling" means being able to provide a full listing
+ * straight out of cache, which we don't in this particular incarnation, but we
+ * take the opportunity to do our own cooking with our cache data, creating the
+ * directory in cache if need be, and checking for cache data sanity.
+ * 
  *
  * @param r Apache request_rec struct.
- * @param entries Ignored
+ * @param pack Ignored
  * @param conf The config structure used to find out cache configuration.
  * @param names Structure containing the directory name
  * @param soptions Flags to use for created entries. Ignored.
  *
- * @return 0 on succes, CA_MISSARG if path is missing, CA_FATAL otherwise.
+ * @return NULL.
  */
-static void* cache_file_opendir(request_rec *r, mu_ent **entries, const mu_config *const conf,
+static void* cache_file_opendir(request_rec *r, mu_pack *const pack, const mu_config *const conf,
 				const mu_ent_names * const names, unsigned long soptions)
 {
 	const char *const path = names->filename;
@@ -342,10 +349,9 @@ typedef struct {
  * This function reads the tags from the cache file
  * and fills in the struct mu_ent fields accordingly.
  *
- * @warning <b>MUST</b> close in file handler on success.
+ * @warning @b MUST close in file handler on success.
  *
  * @param pool Apache pool
- * @param head Head of music entries
  * @param in Not used (except for closing).
  * @param conf MusicIndex configuration paramaters struct.
  * @param names struct names to get the current filename.
@@ -354,10 +360,10 @@ typedef struct {
  * @return When possible, struct mu_ent correctly set up, file stream closed,
  * 		Head pointer otherwise.
  */
-mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	mu_ent *const head, const mu_config *const conf, mu_ent_names *const names)
+static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
+	const mu_config *const conf, mu_ent_names *const names)
 {
-	mu_ent		*p = head;
+	mu_ent		*p = NULL;
 	short 		result = 0;
 	unsigned short	track, posn, flags, cvers = 0;
 	signed short	filetype;
@@ -398,9 +404,7 @@ mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	 * fread()/fwrite() are thread safe, but not fscanf() and fprintf() */
 
 	p = NEW_ENT(r->pool);
-	if (!p)
-		p = head;	/* something failed, return non-bogus data. */
-	else {
+	if (p) {
 		data_buffer = (mi_data_buffer *)malloc(sizeof(mi_data_buffer)); /* This should save some memory */
 		if (data_buffer) {
 			result = fscanf(cache_file, "album: %[^\n]\nartist: %[^\n]\n"
@@ -413,11 +417,10 @@ mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 		
 			/* Check whether the cache is somehow corrupted */
 			if ((result != CACHE_NF) || (cvers != CACHE_VERS)) {	/* fscanf() returns the number of input items assigned */
-				p = head;					/* hopefuly p allocs should be cleaned by apache */
+				p = NULL;					/* hopefuly p allocs should be cleaned by apache */
 			}
 			else {
-				if (!ISBLANK(data_buffer->title))
-					p->title = ap_pstrdup(r->pool, data_buffer->title);
+				p->title = ap_pstrdup(r->pool, data_buffer->title);
 				if (!ISBLANK(data_buffer->album))
 					p->album = ap_pstrdup(r->pool, data_buffer->album);
 				if (!ISBLANK(data_buffer->artist))
@@ -435,14 +438,14 @@ mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			free(data_buffer);
 		}
 		else
-			p = head;	/* something failed, return non-bogus data. p will be cleaned up by apache's GC */
+			p = NULL;	/* something failed, return non-bogus data. p will be cleaned up by apache's GC */
 	}
 
 	/* Explicitely releasing the lock, and closing the file */
 	flock(fileno(cache_file), LOCK_UN);
 	fclose(cache_file);
 
-	if (p != head)
+	if (p)
 		fclose(in);	/* this part of the cache subsystem is (uglily) seen as part of the playlist system,
 				and has to behave as such. This is why we close the input file if we took advantage of it. */
 
@@ -463,10 +466,8 @@ error_out:
  * @param p A mu_ent struct containing the data to store.
  * @param conf The config structure used to find out cache configuration.
  * @param names A names structure in which the file name can be found.
- *
- * @return 0 on success, CA_LOCKED if the file is locked already, CA_FATAL otherwise.
  */
-void cache_file_write(request_rec *r, mu_ent *p, const mu_config *const conf, const mu_ent_names * const names)
+static void cache_file_write(request_rec *r, mu_ent *p, const mu_config *const conf, const mu_ent_names * const names)
 {
 	FILE	*cache_file = NULL;
 
@@ -491,7 +492,7 @@ void cache_file_write(request_rec *r, mu_ent *p, const mu_config *const conf, co
 	fprintf(cache_file, "album: %s\nartist: %s\ntitle: %s\ndate: %hu\n"
 		"track: %hhu\nposn: %hhu\nlength: %hu\nbitrate: %lu\nfreq: %hu\n"
 		"size: %lu\nfiletype: %hi\ngenre: %s\nmtime: %lu\nflags: %hhx\ncvers: %hu\n",
-		BLANKSTR(p->album), BLANKSTR(p->artist), BLANKSTR(p->title), p->date,
+		BLANKSTR(p->album), BLANKSTR(p->artist), p->title, p->date,
 		p->track, p->posn, p->length, p->bitrate, p->freq, p->size, p->filetype,
 		BLANKSTR(p->genre), p->mtime, (p->flags & EF_FLAGSTOSAVE), CACHE_VERS);
 
@@ -513,13 +514,13 @@ static const cache_backend cache_backend_file = {
 	cache_file_write
 };
 
-int cache_file_setup(apr_pool_t *pool, const char *const setup_string, mu_config *const conf)
+int cache_file_setup(cmd_parms *cmd, const char *const setup_string, mu_config *const conf)
 {
 	static const char biniou[] = "file://";
 
 	if (strncmp(biniou, setup_string, 7) == 0) {
 		conf->cache = &cache_backend_file;
-		conf->cache_setup = ap_pstrdup(pool, setup_string+6);
+		conf->cache_setup = ap_pstrdup(cmd->pool, setup_string+6);
 		return 0;
 	}
 
