@@ -2,7 +2,7 @@
  *  html.c
  *  mod_musicindex
  *
- *  $Id: html.c 755 2007-04-29 20:47:50Z varenet $
+ *  $Id: html.c 768 2007-08-22 12:35:16Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 755 $
+ * @version $Revision: 768 $
  * @date 2003-2007
  *
  * That file groups all functions related to HTML and assimilated
@@ -32,7 +32,6 @@
  * @todo Replace text with eye-candy icons wherever possible
  * @todo reduce the size of this file & optimize functions
  * @todo Fix search box collide on long path names
- * @todo Fix translation of form buttons
  */
 
 #include "html.h"
@@ -78,6 +77,8 @@ static const char *const Gcovericns[] = {
  *
  * @todo Sort option for recursive search?
  * @todo Keep sort conf on userside (using cookie(s)).
+ * @todo We have the basis of doing per file permissions in the Select panel stuff
+ *	which also needs to be rewritten and simplified
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param p struct mu_ent.
@@ -97,7 +98,7 @@ list_songs(request_rec *r, const mu_ent *const p, const mu_config *const conf,
 
 	ap_rputs("  <tr class=\"title\">\n", r);
 
-	if ( (customlist != 0) || (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM)) ||
+	if ( (customlist != 0) || (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM | MI_ALLOWTARBALL)) ||
 		((conf->search) && (conf->options & MI_RECURSIVE)))
 		ap_rvputs(r, "   <th class=\"Select\">", _("Select"), "</th>\n", NULL);
 
@@ -221,15 +222,15 @@ list_songs(request_rec *r, const mu_ent *const p, const mu_config *const conf,
 			ap_rputs("  <tr class=\"even\">\n", r);
 
 		/* prepare the "Select" panel, if any */
-		if ((customlist != 0) || (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM)) ||
+		if ((customlist != 0) || (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM | MI_ALLOWTARBALL)) ||
 			((conf->search) && (conf->options & MI_RECURSIVE))) {
 			/* this should save some memory */
 			codeduri = (char *)realloc(codeduri, 1 + ap_base64encode_len(strlen(q->uri)));
 
 			ap_rputs("   <td class=\"Select\">\n", r);
 
-			/* Display checkbox */
-			if (codeduri) {
+			/* Display checkbox - no matter what if we have a customlist, only if stream/tarball allowed otherwise */
+			if (codeduri && (customlist || (conf->options & (MI_ALLOWSTREAM | MI_ALLOWTARBALL)))) {
 				ap_base64encode(codeduri, q->uri, strlen(q->uri));
 				/* we base64 encode the uri to avoid nasty special chars problems */
 				ap_rvputs(r, "    <input type=\"checkbox\" name=\"file\" value=\"",
@@ -374,7 +375,7 @@ static void send_url(request_rec *r, const char *const uri, const char *const co
 		strcat(prefix, "://");
 
 		if (REQUEST_USER(r)) {
-			/* grab the auth credentials base64 encoded */
+			/* grab the auth credentials base64 encoded - apparently only for auth_basic */
 			const char *const auth = ap_table_get(r->headers_in, "Authorization");
 			if (auth) {
 				bufcoded = strrchr(auth, ' ');
