@@ -2,11 +2,11 @@
  *  http.c
  *  mod_musicindex
  *
- *  $Id: http.c 671 2006-07-31 23:37:21Z varenet $
+ *  $Id: http.c 753 2007-04-27 14:04:42Z varenet $
  *
  *  Created by Regis BOUDIN on Sun Jun 13 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
- *  Copyright (c) 2006 Thibaut VARENE
+ *  Copyright (c) 2006-2007 Thibaut VARENE
  *  
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,8 +20,8 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 671 $
- * @date 2004-2006
+ * @version $Revision: 753 $
+ * @date 2004-2007
  *
  * This file is here to deal with HTTP related
  * features which are not directly handled by
@@ -36,10 +36,22 @@
  * so we should be perfectly fine dealing with '&' and friends.
  *
  * We store in the cookie file identifier as follows:
- * <base64_encoded_uri>&"
+ * base64_encoded_uri&"
  * We store the unescaped base64 string in the cookie.
  *
+ * Also note that according to: http://www.15seconds.com/faq/Cookies/388.htm @n
+ * According to the draft specification issued by Netscape Communications, the
+ * limits regarding the size of cookie and space occupied by all cookies is:
+ *  - 4 KB per cookie maximum
+ *  - 300 total cookies, for a total of 1.2 Mbytes maximum
+ *  - 20 cookies accepted from a particular server or domain
+ *
+ * Which means that we don't have infinite space, and can't store big custom
+ * lists. Reports shows that ~50 songs is a max, but if depends on the length
+ * of the path anyway.
+ *
  * @todo Test, test, test.
+ * @todo Use cookie for session identifer, and store custom lists on the server.
  *
  * @bug Beware of "big lists"
  * @todo Store custom list locally, and store only a session reference in the cookie.
@@ -48,6 +60,9 @@
 #include "http.h"
 #include "sort.h"
 #include "playlist.h"
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>	/* atoi */
+#endif
 
 /**
  * Treats the GET arguments
@@ -76,6 +91,7 @@ void treat_get_args(request_rec *r, mu_config *const conf)
 			
 			if (!strcmp(p, "randomdir")) {
 				conf->options |= MI_RANDOMDIR;
+				return;
 			}
 			
 			if (conf->options & MI_ALLOWTARBALL) {
@@ -83,13 +99,14 @@ void treat_get_args(request_rec *r, mu_config *const conf)
 					conf->options |= (MI_DWNLDALL | MI_QUICKPL) ;
 					conf->order[0] = SB_DIR;
 					conf->order[1] = SB_URI;
-					set_fctptrs(conf);
+					continue;
 				}
 			}
 			
 			if (conf->options & MI_ALLOWSTREAM) {
 				if (!strcmp(p, "playall"))
 					conf->options |= MI_STREAMALL;
+					continue;
 			}
 			
 			if (conf->rss_items > 0) {
@@ -98,26 +115,24 @@ void treat_get_args(request_rec *r, mu_config *const conf)
 					conf->order[0] = SB_MTIME;
 					conf->order[1] = SB_URI;
 					conf->options &= ~MI_RECURSIVE;
+					continue;
 				}
 
-				if (!strcmp(p, "podcast")) {
+				else if (!strcmp(p, "podcast")) {
 					conf->options |= MI_RSS|MI_PODCAST;
 					conf->order[0] = SB_MTIME;
 					conf->order[1] = SB_URI;
 					conf->options &= ~MI_RECURSIVE;
+					continue;
 				}
 			}
-			
-			return;
 		}
 		else if (!strncmp(p, "sort=", 5)) {
 			p += 5;
 		/* Temporary workaround : only have one sort param */
-			conf->order[ARG_NUMBER-1] = '\0';
-			for (i = ARG_NUMBER-2; i > 0; i--)
+			for (i = SB_MAX; i > 0; i--)
 				conf->order[i] = conf->order[i-1];
-			conf->order[0] = *p;
-			set_fctptrs(conf);
+			conf->order[0] = (atoi(p) % SB_MAX);	/* avoid out of bound values */
 		}
 		else if (!strncmp(p, "option=", 7)) {
 			p += 7;
@@ -126,7 +141,6 @@ void treat_get_args(request_rec *r, mu_config *const conf)
 			else if (!strcmp(p, "shuffle")) {
 				conf->order[0] = SB_RANDOM;
 				conf->order[1] = SB_DEFAULT;
-				set_fctptrs(conf);
 			}
 			else if (!strcmp(p, "quick"))
 				conf->options |= MI_QUICKPL;
@@ -145,6 +159,11 @@ void treat_get_args(request_rec *r, mu_config *const conf)
  * request. It only sets option flags and the search string when they are
  * needed.
  * 
+ * @warning This function is fairly inefficient as-is. It does a lot of useless checks
+ * and string comparisons, because it will only deal with 1 action at a time, so
+ * as soon as we have one hit, we could give up. We could add 'return's everywhere
+ * but it's pretty ugly as well.
+ *
  * @param r Apache request_rec struct to handle connection details.
  * @param conf MusicIndex configuration paramaters struct.
  */
@@ -186,7 +205,7 @@ void treat_post_args(request_rec *r, mu_config *const conf)
 		}
 		ap_unescape_url(conf->search); /* x-www-form-urlencoded forms */
 	}
-	
+
 	/* Find the sort string */
 	p = strstr(s, "&sort=");
 
@@ -197,14 +216,14 @@ void treat_post_args(request_rec *r, mu_config *const conf)
 	
 	if (p != NULL) {
 		p += 5;
-		conf->order[ARG_NUMBER-1] = '\0';
-		for (i = ARG_NUMBER-2; i > 0; i--)
-			conf->order[i] = conf->order[i-1];
-		conf->order[0] = *p;
-		set_fctptrs(conf);
+		/* The client sends us a complete sort sequence.
+		 * The modulo works around potentially (forged?) out of bound values */
+		for (i = 0; p[i] && (p[i] != '&') && (i < SB_MAX); i++)
+			conf->order[i] = ((p[i]-'`') % SB_MAX);	/* unshift from letters */
 	}
-	
-	/* Find the action string */
+
+	/* Find the action string. We expect to have only one action at a time.
+	 * Otherwise, supplementary actions will be ignored. */
 	p = strstr(s, "&action=");
 
 	if (p != NULL)
@@ -217,64 +236,62 @@ void treat_post_args(request_rec *r, mu_config *const conf)
 	
 		p = ap_getword(r->pool, &p, '&');
 		
-		if (conf->options & MI_ALLOWSTREAM) {
-			if (!strcmp(p, "Play+Selected")) {
-				conf->options |= MI_STREAMLST;
-			} else if (!strcmp(p, "Play+All")) {
-				conf->options |= MI_STREAMALL;
-			} else if (!strcmp(p, "Shuffle+All")) {
-				conf->options |= MI_STREAMALL;
-				conf->order[0] = SB_RANDOM;
-				conf->order[1] = SB_DEFAULT;
-				set_fctptrs(conf);
-			} else if (!strcmp(p, "Remove+from+Playlist")) {
-				conf->options |= MI_COOKIEDELLST;
-			} else if (!strcmp(p, "Clear+Playlist")) {
-				conf->options |= MI_COOKIEPURGE;
-			} else if (!strcmp(p, "Stream+Playlist")) {
-				conf->options |= MI_COOKIESTREAM;
-			} else if (!strcmp(p, "Add+To+Playlist")) {
-				conf->options |= MI_COOKIEADDLST;
-			} else if (!strcmp(p, "Add+All+To+Playlist")) {
-				conf->options |= MI_COOKIEADDALL;
-			}
-			
-			/* In any situation, no recursive directories
-			scan for Cookie operations */
-			if (conf->options & MI_COOKIEOP)
-				conf->options &= ~MI_RECURSIVE;
-		}
-		
-		if (conf->options & MI_ALLOWTARBALL) {
-
-			if (!strcmp(p, "Download+All")) {
-				conf->options |= MI_DWNLDALL;
-			} else if (!strcmp(p, "Download+Selected")) {
-				conf->options |= MI_DWNLDLST;
-			}else if (!strcmp(p, "Download+Playlist")) {
-				conf->options |= MI_COOKIEDWNLD;
-			}
-			
-			/* In any situation, no recursive directories
-			scan for Cookie operations */
-			if (conf->options & MI_COOKIEOP)
-				conf->options &= ~MI_RECURSIVE;
-		}
-		
 		if ((conf->options & MI_ALLOWSEARCH) && conf->search && conf->search[0]) {
 			if (!strcmp(p, "Search")) {
 				if (!conf->cache)
 					conf->options |= MI_QUICKPL;
 			}
-			else if (!strcmp(p, "Recursive+Search")) {
+			else if (!strcmp(p, "RecursiveSearch")) {
 				conf->options |= MI_RECURSIVE;
 				conf->order[0] = SB_DIR;
 				conf->order[1] = SB_URI;
-				set_fctptrs(conf);
 				if (!conf->cache)
 					conf->options |= MI_QUICKPL;
 			}
 		}
+		
+		/* In any situation, no recursive directories
+		scan for Cookie operations */
+		if (conf->options & MI_COOKIEOP)
+			conf->options &= ~MI_RECURSIVE;
+
+		if (conf->options & MI_ALLOWSTREAM) {
+			if (!strcmp(p, "PlaySelected")) {
+				conf->options |= MI_STREAMLST;
+			} else if (!strcmp(p, "PlayAll")) {
+				conf->options |= MI_STREAMALL;
+			} else if (!strcmp(p, "ShuffleAll")) {
+				conf->options |= MI_STREAMALL;
+				conf->order[0] = SB_RANDOM;
+				conf->order[1] = SB_DEFAULT;
+			}
+		}
+		
+		if (conf->options & MI_ALLOWTARBALL) {
+			if (!strcmp(p, "DownloadAll")) {
+				conf->options |= MI_DWNLDALL;
+			} else if (!strcmp(p, "DownloadSelected")) {
+				conf->options |= MI_DWNLDLST;
+			}
+		}
+
+		if ((conf->options & (MI_ALLOWSTREAM|MI_ALLOWTARBALL))) {
+			if (!strcmp(p, "AddToPlaylist")) {
+				conf->options |= MI_COOKIEADDLST;
+			} else if (!strcmp(p, "AddAllToPlaylist")) {
+				conf->options |= MI_COOKIEADDALL;
+			}
+		}
+
+		if (!strcmp(p, "RemoveFromPlaylist")) {
+			conf->options |= MI_COOKIEDELLST;
+		} else if (!strcmp(p, "ClearPlaylist")) {
+			conf->options |= MI_COOKIEPURGE;
+		} else if (!strcmp(p, "StreamPlaylist")) {
+			conf->options |= MI_COOKIESTREAM;
+		} else if (!strcmp(p, "DownloadPlaylist")) {
+			conf->options |= MI_COOKIEDWNLD;
+		}
 	}
 }
 
@@ -331,9 +348,14 @@ static const char *cookie_add(request_rec *r, apr_pool_t *subpool, const mu_conf
  */
 static const char *cookie_addall(request_rec *r, apr_pool_t *subpool, mu_config *const conf)
 {
-	mu_ent		*head = NULL, *custom = NULL;
+	const mu_ent	*custom = NULL;
 	const char 	*new_cookie_string = NULL, *p = NULL;
-	char		*codeduri;
+	char		*codeduri = NULL;
+	mu_pack cookie_pack = {
+		.head = NULL,
+		.dirnb = 0,
+		.filenb = 0,
+	};
 
 	p = ap_table_get(r->headers_in, "Cookie");
 
@@ -342,14 +364,14 @@ static const char *cookie_addall(request_rec *r, apr_pool_t *subpool, mu_config
 	else
 		new_cookie_string = ap_pstrdup(subpool, "playlist=");
 	
-	head = make_music_entry(r, subpool, NULL, conf, NULL, MI_RECURSIVE);
-	head = quicksort(head, NULL, conf);
+	make_music_entry(r, subpool, &cookie_pack, conf, NULL, MI_RECURSIVE);
+	listsort(&cookie_pack, conf->order);
 
-	for (custom = head; custom; custom = custom->next) {
+	for (custom = cookie_pack.head; custom; custom = custom->next) {
 		if (custom->filetype < 0)
 			continue;
 
-		codeduri = (char *)ap_palloc(subpool, 1 + ap_base64encode_len(strlen(custom->uri)));
+		codeduri = (char *)realloc(codeduri, 1 + ap_base64encode_len(strlen(custom->uri)));
 		if (!codeduri)
 			return NULL;
 		ap_base64encode(codeduri, custom->uri, strlen(custom->uri));
@@ -358,6 +380,7 @@ static const char *cookie_addall(request_rec *r, apr_pool_t *subpool, mu_config
 			new_cookie_string = ap_pstrcat(subpool, new_cookie_string, codeduri, "&", NULL);
 	}
 
+	free(codeduri);
 	return new_cookie_string;
 }
 
@@ -414,6 +437,8 @@ static const char *cookie_remove(request_rec *r, apr_pool_t *subpool, const mu_c
 	return new_cookie_string;
 }
 
+/**
+ * @warning What if subpool == r->pool wrt destroy()? */
 void cookie_and_stream_work(request_rec *r, mu_config *const conf)
 {
 	const char *end_string = NULL, *args = NULL;
