@@ -2,7 +2,7 @@
  *  mod_musicindex.c
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.c 756 2007-05-01 19:27:56Z varenet $
+ *  $Id: mod_musicindex.c 769 2007-08-22 12:43:06Z varenet $
  *
  *  Created by Regis BOUDIN on Sat Dec 28 2002.
  *  Copyright (c) 2002-2006 Regis BOUDIN
@@ -40,6 +40,7 @@
  *	- XHTML output.
  *	- Archive download (with libarchive - EXPERIMENTAL).
  *	- RSS and Podcasts feeds generation.
+ *	- Support for auth_basic authentication in URL generation
  *
  * @section dependencies Dependencies
  * 	- <a href="http://httpd.apache.org/download.cgi">Apache</a> (1.3 or 2).
@@ -106,7 +107,7 @@
  * This file is the core of the module. It contains Apache's mandatory stuff.
  *
  * @author Regis Boudin
- * @version $Revision: 756 $
+ * @version $Revision: 769 $
  * @date 2002-2005
  * @date 2007
  *
@@ -395,7 +396,7 @@ static int handle_musicfile(request_rec *r)
 		if (r->header_only)
 			return OK;
 
-		make_music_entry(r, r->pool, &master_pack, conf, NULL, 0);
+		make_music_entry(r, r->pool, &master_pack, conf, NULL, MI_ALLOWSTREAM);
 		send_playlist(r, &master_pack, conf);
 
 		return OK;
