@@ -2,10 +2,10 @@
  *  playlist-mp4.c
  *  mod_musicindex
  *
- *  $Id: playlist-mp4.c 574 2006-03-27 21:56:39Z boudinr $
+ *  $Id: playlist-mp4.c 754 2007-04-29 00:57:53Z varenet $
  *
  *  Created by Thibaut VARENE on Tue Sep  2 2004.
- *  Copyright (c) 2004-2005 Thibaut VARENE
+ *  Copyright (c) 2004-2005,2007 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -18,8 +18,9 @@
  * MP4 files management system.
  *
  * @author Thibaut Varene
- * @version $Revision: 574 $
+ * @version $Revision: 754 $
  * @date 2004-2005
+ * @date 2007
  *
  * This file contains everything needed to produce music entries from
  * MP4 files.
@@ -38,16 +39,16 @@
  * @param filename The filename to check.
  * @todo see if we can just rely on the trackType test.
  *
- * @return FALSE when the extension is correct, TRUE otherwise.
+ * @return TRUE when the extension is correct, FALSE otherwise.
  */
-static short mp4_ext_check(const char *const filename)
+static inline short mp4_ext_check(const char *const filename)
 {
-	char *ext = strrchr(filename, '.');
-	if(ext++ && (!strncasecmp(ext, "mp4", 3)	|| /* official extention */
-		!strncasecmp(ext, "m4a", 3)		|| /* Apple mp4 file */
-		!strncasecmp(ext, "aac", 3))		)  /* old MPEG2/4 extention */
-		return FALSE;
-	return TRUE;
+	const char *const ext = strrchr(filename, '.');
+	if(ext && (!strncasecmp(ext, ".mp4", 4)	|| /* official extention */
+		!strncasecmp(ext, ".m4a", 4)	|| /* Apple mp4 file */
+		!strncasecmp(ext, ".aac", 4))	)  /* old MPEG2/4 extention */
+		return TRUE;
+	return FALSE;
 }
 
 /**
@@ -57,21 +58,20 @@ static short mp4_ext_check(const char *const filename)
  * and fills in the struct mu_ent fields accordingly.
  * The code is based on sample files from mpeg4ip.
  *
+ * @param r Apache request_rec struct to handle log writings (debugging)
  * @param pool Pool
- * @param head Head
  * @param in MP4 file to parse (closed on normal exit)
  * @param conf MusicIndex configuration paramaters struct
  * @param names Names
- * @param r Apache request_rec struct to handle log writings (debugging)
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed.
  *
  * @todo handle old AAC files that use id3 tags.
  */
 mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	mu_ent *const head, const mu_config *const conf, mu_ent_names *const names)
+	const mu_config *const conf, mu_ent_names *const names)
 {
-	mu_ent 			*p = head;
+	mu_ent 			*p = NULL;
 
 	struct stat		filestat;
 	MP4FileHandle		mp4file;
@@ -81,10 +81,9 @@ mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	register unsigned short i;
 	
 	/* XXX We could probably rely on the trackType test only though */
-	if (mp4_ext_check(names->filename))
+	if (!mp4_ext_check(names->filename))
 		goto exit;
 
-#warning "revisit: detect by content/speedup parsing"	
 	/* XXX va savoir ce qui pourrait faire foirer le read, je me souviens plus
 	des heritages en C++ ca me gave. Si une sous fonction chie, la fonction parente chie? */
 	if (!(mp4file = MP4Read(names->filename, 0)))
@@ -100,7 +99,7 @@ mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			p = NEW_ENT(pool);
 			if (p == NULL) {
 				MP4Close(mp4file);
-				return head;
+				return NULL;
 			}
 				
 			p->filetype = FT_MP4;
