@@ -2,7 +2,7 @@
  *  inf.h
  *  mod_musicindex
  *
- *  $Id: sort.h 566 2006-02-04 01:14:20Z boudinr $
+ *  $Id: sort.h 739 2007-04-22 22:25:05Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -19,9 +19,7 @@
 
 #include "mod_musicindex.h"
 
-mu_ent *quicksort(mu_ent *base, mu_ent *end, const mu_config *const conf);
-
-void set_fctptrs(mu_config *const conf);
-
+#define listsort sort_mergesort
+void listsort(mu_pack *const, const unsigned char *const);
 
 #endif	/* INF_H */
